CANCELLED = "CANCELLED"
CANCEL_REQUESTED = "CANCEL_REQUESTED"
RUNNING = "RUNNING"
COMPLETED = "COMPLETED"
FAILED = "FAILED"
READY = "READY"

FINISHED = [COMPLETED, FAILED, CANCELLED]
ACTIVE = [READY, RUNNING]
UNSUCCESSFUL = [FAILED, CANCELLED]
ALL = [CANCELLED, CANCEL_REQUESTED, RUNNING, COMPLETED, FAILED, READY]
